package Tapper::CLI;
# git description: v4.0.1-57-gee26fc8

BEGIN {
  $Tapper::CLI::AUTHORITY = 'cpan:AMD';
}
{
  $Tapper::CLI::VERSION = '4.1.0';
}
# ABSTRACT: Tapper - tapper-* commandline tools

use warnings;
use strict;

1; # End of Tapper::CLI

__END__
=pod

=encoding utf-8

=head1 NAME

Tapper::CLI - Tapper - tapper-* commandline tools

=head1 AUTHOR

AMD OSRC Tapper Team <tapper@amd64.org>

=head1 COPYRIGHT AND LICENSE

This software is Copyright (c) 2012 by Advanced Micro Devices, Inc..

This is free software, licensed under:

  The (two-clause) FreeBSD License

=cut

